/*---------------------------------------------------------------------------\
| PicCore: Framework for the PIC18F458
|
| File:                 adc.c          All adc Code
|
| Global Routines:      initialize_adc
|                       adc_isr
|
| Local Routines:       None
|
| Discussion:           Sure.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created. Again.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes and local #defines
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the project (includes other includes!)


//------------------------------------------------------------------------
//  Global Variables
//------------------------------------------------------------------------

// None

//------------------------------------------------------------------------
//  Module-wide variables
//------------------------------------------------------------------------

// None


/*---------------------------------------------------------------------------\
| Function:     initialize_adc
|
| Purpose:      Guess.
|
| Discussion:   Huh?
|
| Expects:      All interupts to be off (after all, we were just reset).
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void initialize_adc (enum ADC_INIT_MASK adc_options)
{
    // Turn the ADC peripheral on or off

    if (adc_options & ADC_ENABLE_MASK)
        ADON = 1;
    else
        ADON = 0;

    // Set the channel: ~(ADC_CHANNEL_MASK) = 0b11000111

    ADCON0 = (ADCON0 & ~(ADC_CHANNEL_MASK)) | (adc_options & ADC_CHANNEL_MASK);

    // Set the result justification

    if (adc_options & ADC_JUSTIFY_MASK)
        ADFM = 1;
    else
        ADFM = 0;

    // Set the port pins appropriately

    switch (adc_options & ADC_PORT_MASK)
    {

        case ADC_PORT_OFF:
            ADCON1 = (ADCON1 & 0b11110000) | 0b00000110;    // All digital pins
            break;

        case ADC_PORT_1_CH:
            ADCON1 = (ADCON1 & 0b11110000) | 0b00001110;    // RA0
            break;

        case ADC_PORT_3_CH:
            ADCON1 = (ADCON1 & 0b11110000) | 0b00000100;    // RA0/1/2
            break;

        case ADC_PORT_5_CH:
            ADCON1 = (ADCON1 & 0b11110000) | 0b00000110;    // RA0/1/2/3/5,RE0
            break;

        default:
            ADCON1 = (ADCON1 & 0b11110000) | 0b00000110;    // All digital pins
            ThrowWarning(ADC_BAD_PIN_SETUP);
    }

    // Set the ADC clock divider automatically depending on the frequency
    // of the PIC crystal

#if PROCESSOR_CLOCK_FREQ == 20

    ADCS2 = 0;
    ADCS1 = 1;
    ADCS0 = 0;

#elif PROCESSOR_CLOCK_FREQ == 40

    ADCS2 = 1;
    ADCS1 = 1;
    ADCS0 = 0;

#else
    #error adc.c: Hey! PROCESSOR_CLOCK_FREQ in config.h must be 20 or 40.
#endif

    // Set the ADC interrupt priority based on settings in config.h

    #if INTS_ADC_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    set_adc_ints_high();
    #else
    set_adc_ints_low();
    #endif

    // Set the ADC interrupt enable

    if (adc_options & ADC_INTS_MASK)
        enable_adc_interrupts();
    else
        disable_adc_interrupts();
}


/*---------------------------------------------------------------------------\
| Function:     adc_isr
|
| Purpose:      Handle the ADC interrupt.
|
| Discussion:   Clears the Timer2 interrupt and if an interrupt handler exists
|               (the function pointer is non-null) then it calls the handler.
|               Note that if the Timer 2 interrupts are turned off, this
|               function goes away (and thus never has the opportunity to
|               vector off to la-la land.
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Added application specific interrup handler via function ptr.
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

#if INTS_ADC_ISR_USED == INTS_CORE_ISR_ENABLED

void adc_isr (void)
{
    clear_adc_interrupt();

    // If an application specific interrupt handler is defined,
    // then call it. See init_int.h.

    #if INTS_ADC_APPLICATION_HOOK == INTS_HOOK_ENABLED
    adc_application_handler();
    #endif
}

#endif



/*---------------------------------------------------------------------------\
| Function:     switch_adc_channels
|
| Purpose:      Switch the ADC channel number
|
| Discussion:   Does what you expect it to do.
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/02/2003 | Created.
\---------------------------------------------------------------------------*/

void switch_adc_channels (uint8_t channel)
{
    // Switch channels to the channel pointed to by "channel". Make sure
    // the value is not greater than 7 (0b00000111) and that the other
    // bits in ADCON0 are not disturbed

    ADCON0 = (ADCON0 & 0b11000111) | ((channel << 3) & 0b00111000);
}
