/*---------------------------------------------------------------------------\
| Project:  Generic LV2 PIC18F458 CAN node firmware
|
| File:     error.h
|
| Descr.:   Includes for all exception handling
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _PICCORE_ERROR_H   // Don't include more than once (last line = #endif)
#define _PICCORE_ERROR_H

//------------------------------------------------------------------------
// Typedefs, enums, etc.
//------------------------------------------------------------------------

enum WARNING_MESSAGES
{
    WARN_SERIAL_BITRATE = 1,        // 01
    SERIAL_TXQUEUE_OVERRUN,
    SERIAL_RXQUEUE_OVERRUN,
    WARN_TIMER2_RATE,
    WARN_ISR_UNKNOWN_HPI,           // 05
    WARN_ISR_UNKNOWN_LPI,
    WARN_CAN_BITRATE,
    WARN_CAN_FILTER_NUM,
    WARN_CAN_MASK_NUM,
    WARN_CAN_BAD_TX_DLC,            // 10
    WARN_CAN_BAD_TX_ID,

    WARN_CAN_TXB2_NOT_EMPTY,        // 12
    WARN_CAN_TX2_QUEUE_EMPTY,
    WARN_CAN_TX2_QUEUE_OVERFLOW,

    WARN_CAN_TX1_QUEUE_OVERFLOW,    // 15
    WARN_CAN_TXB1_NOT_EMPTY,
    WARN_CAN_TX1_QUEUE_EMPTY,

    WARN_CAN_TXB0_NOT_EMPTY,        // 18
    WARN_CAN_TX0_QUEUE_EMPTY,
    WARN_CAN_TX0_QUEUE_OVERFLOW,

    CAN_RX_QUEUE_OVERRUN_RXB0,      // 21
    CAN_RX_QUEUE_OVERRUN_RXB1,
    CAN_RX0ISR_BAD_DLC,
    CAN_RX1ISR_BAD_DLC,

    ADC_BAD_PIN_SETUP           // 25
};


//------------------------------------------------------------------------
// #defines
//------------------------------------------------------------------------

#define WARNINGLOGSIZE  64

//------------------------------------------------------------------------
// Function declarations
//------------------------------------------------------------------------

void ThrowWarning       (enum WARNING_MESSAGES warningMessage);
void ThrowWarning_HP    (enum WARNING_MESSAGES warningMessage);
void ThrowWarning_LP    (enum WARNING_MESSAGES warningMessage);


#define  ErrorLedOn()       RE1 = 1

//------------------------------------------------------------------------
// Global variable declarations (extern)
//------------------------------------------------------------------------

#endif              // _PICCORE_ERROR_H





