/*---------------------------------------------------------------------------\
| Project:  Generic LV2 PIC18F458 CAN node firmware
|
| File:     ports.h
|
| Descr.:   Includes for the ports.c module
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _PORTS_H    // Don't include more than once (last line = #endif)
#define _PORTS_H

//------------------------------------------------------------------------
// Typedefs
//------------------------------------------------------------------------


// None


//------------------------------------------------------------------------
// #defines
//------------------------------------------------------------------------

#define PORTS_PORTB_PULLUP_MASK     0b00000001
#define PORTS_PORTB_PULLUPS_ON      0b00000001
#define PORTS_PORTB_PULLUPS_OFF     0b00000000

#define PORTS_INT0_EDGE_MASK        0b00000010
#define PORTS_INT0_RISING_EDGE      0b00000010
#define PORTS_INT0_FALLING_EDGE     0b00000000

#define PORTS_INT1_EDGE_MASK        0b00000100
#define PORTS_INT1_RISING_EDGE      0b00000100
#define PORTS_INT1_FALLING_EDGE     0b00000000

#define PORTS_INT0_INTS_MASK        0b00001000
#define PORTS_INT0_INTS_ENABLED     0b00001000
#define PORTS_INT0_INTS_DISABLED    0b00000000

#define PORTS_INT1_INTS_MASK        0b00010000
#define PORTS_INT1_INTS_ENABLED     0b00010000
#define PORTS_INT1_INTS_DISABLED    0b00000000

//------------------------------------------------------------------------
// #defined "inline functions" (PICC18 does not allow inline functions)
//------------------------------------------------------------------------

#define disable_int0_ints()             INT0IE = 0
#define enable_int0_ints()              INT0IE = 1

#define disable_int1_ints()             INT1IE = 0
#define enable_int1_ints()              INT1IE = 1

#define disable_port_b_ioc_ints()       RBIE = 0
#define enable_port_b_ioc_ints()        RBIE = 1

#define clear_int0_interrupt()          INT0IF = 0
#define clear_int1_interrupt()          INT1IF = 0
#define clear_port_b_ioc_interrupt()    RBIF = 0

//------------------------------------------------------------------------
// Function declarations
//------------------------------------------------------------------------

void InitializePorts ( uint8_t port_a_latch,    uint8_t port_a_direction,
                       uint8_t port_b_latch,    uint8_t port_b_direction,
                       uint8_t port_c_latch,    uint8_t port_c_direction,
                       uint8_t port_d_latch,    uint8_t port_d_direction,
                       uint8_t port_e_latch,    uint8_t port_e_direction,
                       uint8_t port_options );

// INT0 declarations (set in config.h)

#if INTS_INT0_ISR_USED == INTS_CORE_ISR_ENABLED
void int0_isr (void);
#endif

#if INTS_INT0_APPLICATION_HOOK == INTS_HOOK_ENABLED
void int0_application_handler (void);
#endif

// INT1 declarations (set in config.h)

#if INTS_INT1_ISR_USED == INTS_CORE_ISR_ENABLED
void int1_isr (void);
#endif

#if INTS_INT1_APPLICATION_HOOK == INTS_HOOK_ENABLED
void int1_application_handler (void);
#endif

// Port B Interrupt On Change (PBIOC) declarations (set in config.h)

#if INTS_PBIOC_ISR_USED == INTS_CORE_ISR_ENABLED
void port_b_ioc_isr (void);
#endif

#if INTS_PBIOC_APPLICATION_HOOK == INTS_HOOK_ENABLED
void port_b_ioc_application_handler (void);
#endif


//------------------------------------------------------------------------
// Module-wide variable declarations
//------------------------------------------------------------------------


// None


#endif              // _PORTS_H




