/*---------------------------------------------------------------------------\
| Project:  Generic LV2 PIC18F458 CAN node firmware
|
| File:     timer2.h
|
| Descr.:   Includes for the timer2.c module
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _TIMER2_H   // Don't include more than once (last line = #endif)
#define _TIMER2_H

//------------------------------------------------------------------------
// Typedefs, enums, etc.
//------------------------------------------------------------------------

enum TIMER2_INIT_MASK
{
    TIMER2_ENABLE_INTS          = 0b10000000,
    TIMER2_DISABLE_INTS         = 0b00000000,

    TIMER2_OFF                  = 0b00000000,   // ** ASSUMES 20MHZ **
    TIMER2_250HZ                = 0b00000001,   // ** ASSUMES 20MHZ **
    TIMER2_500HZ                = 0b00000010,   // ** ASSUMES 20MHZ **
    TIMER2_1KHZ                 = 0b00000011,   // ** ASSUMES 20MHZ **
    TIMER2_2KHZ                 = 0b00000100,   // ** ASSUMES 20MHZ **
    TIMER2_2_5KHZ               = 0b00000101,   // ** ASSUMES 20MHZ **
    TIMER2_5KHZ                 = 0b00000110,   // ** ASSUMES 20MHZ **
    TIMER2_10KHZ                = 0b00000111,   // ** ASSUMES 20MHZ **

};


//------------------------------------------------------------------------
// #defines
//------------------------------------------------------------------------

// Clear the Timer2 interrupt flag

#define clear_timer2_interrupt()    TMR2IF = 0

#define disable_timer2_ints()       TMR2IE = 0
#define enable_timer2_ints()        TMR2IE = 1

#define set_timer2_ints_low()       TMR2IP = 0
#define set_timer2_ints_high()      TMR2IP = 1



//------------------------------------------------------------------------
// Function declarations
//------------------------------------------------------------------------

void initialize_timer2 (enum TIMER2_INIT_MASK options_mask);

void set_timer2_freq (enum TIMER2_INIT_MASK options_mask);

#if INTS_TIMER2_ISR_USED == INTS_CORE_ISR_ENABLED
void timer2_isr (void);
#endif

#if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED
void timer2_application_handler (void);
#endif



//------------------------------------------------------------------------
// Global variable declarations (extern)
//------------------------------------------------------------------------

// None

#endif              // _TIMER2_H





